@@ -18,24 +18,40 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 #include	<config.h>
+#include	<stdio.h>	/* for FILE * in struct cdb */
+#include	<string.h>
 #include	<stdlib.h>
+#include	<unistd.h>
+#include	<errno.h>
+/* support platforms that don't yet conform to C99 */
+#if	HAVE_STDINT_H
+#include	<stdint.h>
+#elif	HAVE_INTTYPES_H
+#include	<inttypes.h>
+#else
+#error No stdint.h or inttypes.h found.
+#endif
 #include	"cdb.h"
-#include	"buffer.h"
+#include	"cdb_hash.h"
 #include	"cdb_make.h"
-#include	"uint32.h"
+#include	"elcerror.h"
+#include	"elcerror_p.h"
 
 int cdb_put(struct cdb_make *c,
 	    unsigned char *key, size_t klen, void *val, size_t vlen)
 {
     int			i;
-    uint32		h;
+    uint32_t		h;
 
     if (cdb_make_addbegin(c, klen, vlen) == -1)
       return -1;
-    buffer_put(&c->b, key, klen);
+    if (fwrite((void *) key, klen, 1, c->fout) < 1)
+      DIE("writing to current database");
     for (i = 0, h = CDB_HASHSTART; i < klen; ++i)
       h	 = cdb_hashadd(h, key[i]);
-    buffer_put(&c->b, val, vlen);
+    if (fwrite(val, vlen, 1, c->fout) < 1)
+      DIE("writing to current database");
+      
     if (cdb_make_addend(c, klen, vlen, h) == -1)
       return -1;
 
