@@ -18,21 +18,29 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 #include	<config.h>
+#include	<stdio.h>	/* for FILE * in struct cdb */
 #include	<stdlib.h>
 #include	<unistd.h>
 #include	<errno.h>
+/* support platforms that don't yet conform to C99 */
+#if	HAVE_STDINT_H
+#include	<stdint.h>
+#elif	HAVE_INTTYPES_H
+#include	<inttypes.h>
+#else
+#error No stdint.h or inttypes.h found.
+#endif
 #include	"cdb.h"
-#include	"buffer.h"
 #include	"cdb_make.h"
-#include	"uint32.h"
 #include	"elcerror.h"
 #include	"cdb_seq.h"
+#include	"packint.h"
 
 #define	CDB_HEADER_SIZE	(2048)
 
 /* endian issues will bite us if we don't call uint32_unpack when
  * reading numbers from the cdb file, so we do it here. */
-static int read_uint32(int fd, uint32 *n)
+static int read_uint32(int fd, uint32_t *n)
 {
     unsigned char	buf[4];
     ssize_t		err;
@@ -42,12 +50,12 @@ static int read_uint32(int fd, uint32 *n)
     else if (err == 0)
       return 1;
     else if (err < sizeof(buf)) {
-      /* Error: could not read a whole uint32 */
+      /* Error: could not read a whole uint32_t */
       errno	 = EIO;		/* ENODATA not on FreeBSD */
       return -1;
     }
     
-    uint32_unpack(buf, n);
+    UINT32_UNPACK(buf, n);
 
     return 0;
 }
@@ -89,7 +97,7 @@ int cdb_seq_eod(cdb_seq *c)
 int cdb_seq_sizes(cdb_seq *c, size_t *klen, size_t *vlen)
 {
     ssize_t	err;
-    uint32	n;
+    uint32_t	n;
 
     if ( (err = read_uint32(c->fd, &n)) == -1)
       return -1;
