@@ -22,7 +22,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 typedef struct cdb_seq {
   int		fd;
-  uint32	eod;		/* end of data */
+  uint32_t	eod;		/* end of data */
 } cdb_seq;
 
 int cdb_seq_start(cdb_seq *c);
