@@ -1,5 +1,5 @@
 /*
-hashtbl
+hashtbl - a handy and fast, free hash table implementation
 Copyright (C) 2001 Ed Cashin
 
 This program is free software; you can redistribute it and/or
@@ -39,9 +39,23 @@ typedef struct elc_hashtbl_struct {
 /* use zero capacity for hashtbl's default capacity */
 void hashtbl_init(hashtbl_t *h, size_t capacity);
 void hashtbl_destroy(hashtbl_t *h);
-void hashtbl_free(hashtbl_t *h);
+
+/* free all nodes in hash table, calling *op for each one's data if
+ * op is not NULL.  This gives caller a way to clean up a complex data
+ * type that is pointed to by a hashtbl element. */
+void hashtbl_free(hashtbl_t *h,
+		  void *(*op)(void *elem, void *data),
+		  void *data);
+
 void *hashtbl_store(hashtbl_t *h, const char *key, size_t keylen, void *data);
 void *hashtbl_remove(hashtbl_t *h, const char *key, size_t keylen);
 void *hashtbl_lookup(hashtbl_t *h, const char *key, size_t keylen);
 
+/* run the function, "op", on each element in the hash table, passing along
+ * to op a data pointer ("data") supplied by the caller
+ */
+void hashtbl_foreach(hashtbl_t *h,
+		     void *(*op)(void *elem, void *data),
+		     void *data);
+
 #endif
