@@ -26,7 +26,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	<fcntl.h>
 #include	<limits.h>
 #include	<errno.h>
-#include	<openssl/sha.h>
+/* support platforms that don't yet conform to C99 */
+#if	HAVE_STDINT_H
+#include	<stdint.h>
+#elif	HAVE_INTTYPES_H
+#include	<inttypes.h>
+#else
+#error No stdint.h or inttypes.h found.
+#endif
+#include	"gnupg/sha1.h"
 #include	"cdb.h"
 #include	"cdb_make.h"
 #include	"cdb_seq.h"
