@@ -24,6 +24,8 @@ typedef enum ruleflag {
   /* prefix rules */
   RULE_IGNORE		 = 0x01,
   RULE_NOCHILDREN	 = 0x02,
+  RULE_NOINHERIT	 = 0x04, /* true if this file has a non-inheriting
+				  * checkset */
 
   /* suffix rules */
   RULE_SUM		 = 0x04,
